# -*- coding: utf-8 -*-
from django import get_version

default_app_config = 'model_revision.apps.ModelRevisionConfig'

__all__ = [

]

VERSION = (0, 0, 3, 'alpha', 0)

__version__ = get_version(VERSION)
